import Query from 'rollun-ts-rql/dist/Query';
import Select from 'rollun-ts-rql/dist/nodes/Select';
import Sort from 'rollun-ts-rql/dist/nodes/Sort';
import AbstractLogicalNode from 'rollun-ts-rql/dist/nodes/logicalNodes/AbstractLogicalNode';
import DialogController from './DialogController';
export var QueryNodeNames;
(function (QueryNodeNames) {
    QueryNodeNames["select"] = "select";
    QueryNodeNames["sort"] = "sort";
    QueryNodeNames["limit"] = "limit";
    QueryNodeNames["query"] = "query";
})(QueryNodeNames || (QueryNodeNames = {}));
export var QueryEditorDialogNames;
(function (QueryEditorDialogNames) {
    QueryEditorDialogNames["addNewNodeDialog"] = "addNewNodeDialog";
})(QueryEditorDialogNames || (QueryEditorDialogNames = {}));
export default class QueryEditorContext {
    constructor(invalidator) {
        this._invalidator = invalidator;
        this._fieldNames = [];
        const initialStatus = {};
        initialStatus[QueryEditorDialogNames.addNewNodeDialog] = { shown: false };
        this._dialogs = new DialogController(invalidator, initialStatus);
    }
    set onApplyQuery(value) {
        this._onApplyQuery = value;
    }
    get onApplyQuery() {
        return this._onApplyQuery;
    }
    get query() {
        return this._query;
    }
    set query(query) {
        this._query = query;
        this._invalidator();
    }
    get fieldNames() {
        return this._fieldNames;
    }
    set fieldNames(value) {
        if (value.length > this._fieldNames.length) {
            this._fieldNames = value;
            this._invalidator();
        }
    }
    get dialogs() {
        return this._dialogs;
    }
    applyQuery(query) {
        this._onApplyQuery(query);
    }
    setQueryNode(node) {
        const oldQuery = this.query;
        this._query = new Query({
            select: oldQuery.selectNode,
            sort: oldQuery.sortNode,
            limit: oldQuery.limitNode,
            query: node
        });
        this._invalidator();
    }
    setSelectNode(node) {
        const oldQuery = this.query;
        this._query = new Query({
            select: node,
            sort: oldQuery.sortNode,
            limit: oldQuery.limitNode,
            query: oldQuery.queryNode
        });
        this._invalidator();
    }
    setSortNode(node) {
        const oldQuery = this.query;
        this._query = new Query({
            select: oldQuery.selectNode,
            sort: node,
            limit: oldQuery.limitNode,
            query: oldQuery.queryNode
        });
        this._invalidator();
    }
    removeNode(queryNodeName) {
        const oldQuery = this.query;
        switch (true) {
            case queryNodeName === QueryNodeNames.select: {
                this._query = new Query({
                    sort: oldQuery.sortNode,
                    limit: oldQuery.limitNode,
                    query: oldQuery.queryNode
                });
                break;
            }
            case queryNodeName === QueryNodeNames.sort: {
                this._query = new Query({
                    select: oldQuery.selectNode,
                    limit: oldQuery.limitNode,
                    query: oldQuery.queryNode
                });
                break;
            }
            case queryNodeName === QueryNodeNames.limit: {
                this._query = new Query({
                    select: oldQuery.selectNode,
                    sort: oldQuery.sortNode,
                    query: oldQuery.queryNode
                });
                break;
            }
            case queryNodeName === QueryNodeNames.query: {
                this._query = new Query({
                    select: oldQuery.selectNode,
                    sort: oldQuery.sortNode,
                    limit: oldQuery.limitNode,
                });
                break;
            }
        }
        this._invalidator();
    }
    removeFieldFromNode(fieldName, nodeName) {
        const oldQuery = this.query;
        switch (nodeName) {
            case QueryNodeNames.select:
                const fieldNameIndex = oldQuery.selectNode.fields.indexOf(fieldName);
                if (fieldNameIndex !== -1) {
                    oldQuery.selectNode.fields.splice(fieldNameIndex, 1);
                    const newFields = [...oldQuery.selectNode.fields];
                    this.setSelectNode(new Select(newFields));
                }
                break;
            case QueryNodeNames.sort:
                if (oldQuery.sortNode.sortOptions.hasOwnProperty(fieldName)) {
                    delete oldQuery.sortNode.sortOptions[fieldName];
                    const newSortOptions = Object.assign({}, oldQuery.sortNode.sortOptions);
                    this.setSortNode(new Sort(newSortOptions));
                }
                break;
        }
        this._invalidator();
    }
    getNodeForPath(path) {
        const query = this.query;
        let currentNode = query.queryNode;
        path.forEach((pathFragment, index) => {
            if (index === 0) {
                return;
            }
            if (currentNode instanceof AbstractLogicalNode) {
                currentNode = currentNode.subNodes[pathFragment];
            }
            else {
                throw new Error(`Invalid path: ${path.join(' ')}`);
            }
        });
        return currentNode;
    }
    hasNodeForPath(path) {
        try {
            this.getNodeForPath(path);
            return true;
        }
        catch (error) {
            return false;
        }
    }
    removeNodeForPath(path) {
        if (path.length === 1 && path[0] === 0) {
            this.removeNode(QueryNodeNames.query);
        }
        else {
            const parentNodePath = path.slice(0, path.length - 1);
            const parentNode = this.getNodeForPath(parentNodePath);
            if (parentNode instanceof AbstractLogicalNode) {
                parentNode.removeNode(path[path.length - 1]);
                this._invalidator();
            }
            else {
                throw new Error(`Node with path ${path.join(' ')} doesnt have children`);
            }
        }
    }
    addNodeForPath(node, path) {
        const parentNodePath = path.slice(0, path.length - 1);
        const parentNode = this.getNodeForPath(parentNodePath);
        if (parentNode instanceof AbstractLogicalNode) {
            parentNode.addNode(node);
            this._invalidator();
        }
        else {
            throw new Error(`Node with path ${path.join(' ')} can't have children`);
        }
    }
}
//# sourceMappingURL=QueryEditorContext.mjs.map